@@ -44,10 +44,12 @@ void compute_fragmentation_metric(void)
 void *umm_info(void *ptr, bool force) {
     uint16_t blockNo = 0;
 
+    UMM_CRITICAL_DECL(id_info);
+
     UMM_CHECK_INITIALIZED();
 
     /* Protect the critical section... */
-    UMM_CRITICAL_ENTRY();
+    UMM_CRITICAL_ENTRY(id_info);
 
     /*
      * Clear out all of the entries in the ummHeapInfo structure before doing
@@ -105,7 +107,7 @@ void *umm_info(void *ptr, bool force) {
             if (ptr == &UMM_BLOCK(blockNo)) {
 
                 /* Release the critical section... */
-                UMM_CRITICAL_EXIT();
+                UMM_CRITICAL_EXIT(id_info);
 
                 return ptr;
             }
@@ -163,7 +165,7 @@ void *umm_info(void *ptr, bool force) {
     DBGLOG_FORCE(force, "+--------------------------------------------------------------+\n");
 
     /* Release the critical section... */
-    UMM_CRITICAL_EXIT();
+    UMM_CRITICAL_EXIT(id_info);
 
     return NULL;
 }
