@@ -135,6 +135,24 @@ static uint16_t umm_blocks(size_t size) {
      * When a block removed from the free list, the space used by the free
      * pointers is available for data. That's what the first calculation
      * of size is doing.
+     *
+     * We don't check for the special case of (size == 0) here as this needs
+     * special handling in the caller depending on context. For example when we
+     * realloc() a block to size 0 it should simply be freed.
+     *
+     * We do NOT need to check for allocating more blocks than the heap can
+     * possibly hold - the allocator figures this out for us.
+     *
+     * There are only two cases left to consider:
+     *
+     * 1. (size <= body)  Obviously this is just one block
+     * 2. (blocks > (2^15)) This should return ((2^15)) to force a
+     *                      failure when the allocator runs
+     *
+     * If the requested size is greater that 32677-2 blocks (max block index
+     * minus the overhead of the top and bottom bookkeeping blocks) then we
+     * will return an incorrectly truncated value when the result is cast to
+     * a uint16_t.
      */
 
     if (size <= (sizeof(((umm_block *)0)->body))) {
@@ -143,12 +161,33 @@ static uint16_t umm_blocks(size_t size) {
 
     /*
      * If it's for more than that, then we need to figure out the number of
-     * additional whole blocks the size of an umm_block are required.
+     * additional whole blocks the size of an umm_block are required, so
+     * reduce the size request by the number of bytes in the body of the
+     * first block.
      */
 
-    size -= (1 + (sizeof(((umm_block *)0)->body)));
+    size -= (sizeof(((umm_block *)0)->body));
 
-    return 2 + size / (UMM_BLOCKSIZE);
+    /* NOTE WELL that we take advantage of the fact that INT16_MAX is the
+     * number of blocks that we can index in 15 bits :-)
+     *
+     * The below expression looks wierd, but it's right. Assuming body
+     * size of 4 bytes and a block size of 8 bytes:
+     *
+     * BYTES (BYTES-BODY) (BYTES-BODY-1)/BLOCKSIZE BLOCKS
+     *     1        n/a                        n/a      1
+     *     5          1                          0      2
+     *    12          8                          0      2
+     *    13          9                          1      3
+     */
+
+    size_t blocks = (2 + ((size-1) / (UMM_BLOCKSIZE)));
+
+    if (blocks > (INT16_MAX)) {
+        blocks = INT16_MAX;
+    }
+
+    return (uint16_t)blocks;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -305,7 +344,7 @@ void umm_init(void) {
 
 /* ------------------------------------------------------------------------
  * Must be called only from within critical sections guarded by
- * UMM_CRITICAL_ENTRY() and UMM_CRITICAL_EXIT().
+ * UMM_CRITICAL_ENTRY(id) and UMM_CRITICAL_EXIT(id).
  */
 
 static void umm_free_core(void *ptr) {
@@ -323,7 +362,7 @@ static void umm_free_core(void *ptr) {
 
     /* Figure out which block we're in. Note the use of truncated division... */
 
-    c = (((void *)ptr) - (void *)(&(UMM_HEAP[0]))) / UMM_BLOCKSIZE;
+    c = (((uint8_t *)ptr) - (uint8_t *)(&(UMM_HEAP[0]))) / UMM_BLOCKSIZE;
 
     DBGLOG_DEBUG("Freeing block %6i\n", c);
 
@@ -359,6 +398,7 @@ static void umm_free_core(void *ptr) {
 /* ------------------------------------------------------------------------ */
 
 void umm_free(void *ptr) {
+    UMM_CRITICAL_DECL(id_free);
 
     UMM_CHECK_INITIALIZED();
 
@@ -372,16 +412,16 @@ void umm_free(void *ptr) {
 
     /* Free the memory withing a protected critical section */
 
-    UMM_CRITICAL_ENTRY();
+    UMM_CRITICAL_ENTRY(id_free);
 
     umm_free_core(ptr);
 
-    UMM_CRITICAL_EXIT();
+    UMM_CRITICAL_EXIT(id_free);
 }
 
 /* ------------------------------------------------------------------------
  * Must be called only from within critical sections guarded by
- * UMM_CRITICAL_ENTRY() and UMM_CRITICAL_EXIT().
+ * UMM_CRITICAL_ENTRY(id) and UMM_CRITICAL_EXIT(id).
  */
 
 static void *umm_malloc_core(size_t size) {
@@ -496,6 +536,7 @@ static void *umm_malloc_core(size_t size) {
 /* ------------------------------------------------------------------------ */
 
 void *umm_malloc(size_t size) {
+    UMM_CRITICAL_DECL(id_malloc);
 
     void *ptr = NULL;
 
@@ -516,11 +557,11 @@ void *umm_malloc(size_t size) {
 
     /* Allocate the memory withing a protected critical section */
 
-    UMM_CRITICAL_ENTRY();
+    UMM_CRITICAL_ENTRY(id_malloc);
 
     ptr = umm_malloc_core(size);
 
-    UMM_CRITICAL_EXIT();
+    UMM_CRITICAL_EXIT(id_malloc);
 
     return ptr;
 }
@@ -528,6 +569,7 @@ void *umm_malloc(size_t size) {
 /* ------------------------------------------------------------------------ */
 
 void *umm_realloc(void *ptr, size_t size) {
+    UMM_CRITICAL_DECL(id_realloc);
 
     uint16_t blocks;
     uint16_t blockSize;
@@ -581,7 +623,7 @@ void *umm_realloc(void *ptr, size_t size) {
 
     /* Figure out which block we're in. Note the use of truncated division... */
 
-    c = (((void *)ptr) - (void *)(&(UMM_HEAP[0]))) / UMM_BLOCKSIZE;
+    c = (((uint8_t *)ptr) - (uint8_t *)(&(UMM_HEAP[0]))) / UMM_BLOCKSIZE;
 
     /* Figure out how big this block is ... the free bit is not set :-) */
 
@@ -592,7 +634,7 @@ void *umm_realloc(void *ptr, size_t size) {
     curSize = (blockSize * UMM_BLOCKSIZE) - (sizeof(((umm_block *)0)->header));
 
     /* Protect the critical section... */
-    UMM_CRITICAL_ENTRY();
+    UMM_CRITICAL_ENTRY(id_realloc);
 
     /* Now figure out if the previous and/or next blocks are free as well as
      * their sizes - this will help us to minimize special code later when we
@@ -709,7 +751,7 @@ void *umm_realloc(void *ptr, size_t size) {
     }
 
     /* Release the critical section... */
-    UMM_CRITICAL_EXIT();
+    UMM_CRITICAL_EXIT(id_realloc);
 
     return ptr;
 }
